/*!
  @file           IFRConversion_ABAPStreamConverter.cpp
  @author         D039759
  @ingroup        IFRConversion_ABAPStreamConverter
  @brief          Implements a class for processing long descriptors
  @see

\if EMIT_LICENCE

    ========== licence begin  GPL
    Copyright (c) 2001-2004 SAP AG

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    ========== licence end


\endif
*/

#include "precom/lcstream.h"
#include "Interfaces/Runtime/IFR_Common.h"
#include "Interfaces/Runtime/Conversion/IFRConversion_ABAPStreamConverter.h"

#define DBUG_CLINK_METHOD_ENTER(x,y) DBUG_CONTEXT_METHOD_ENTER(x, y, &clink)

//----------------------------------------------------------------------
IFRConversion_ABAPStreamConverter::IFRConversion_ABAPStreamConverter(IFR_ShortInfo &shortinfo,
                                                             SAPDBMem_IRawAllocator& allocator)
:IFRConversion_Converter(shortinfo, allocator)
{}

//----------------------------------------------------------------------
IFRConversion_ABAPStreamConverter::~IFRConversion_ABAPStreamConverter()
{}

//----------------------------------------------------------------------
IFR_Retcode
IFRConversion_ABAPStreamConverter::translateABAPStreamInput(IFRPacket_DataPart& datapart,
                                                            void               *streamDescriptor,
                                                            IFR_Length          datalength,
                                                            IFR_Length         *lengthindicator,
                                                            IFR_Int4            dataoffset,
                                                            IFR_ConnectionItem& clink)
{
    DBUG_CONTEXT_METHOD_ENTER(IFRConversion_ABAPStreamConverter, translateABAPStreamInput, &clink);

    SQLStreamDesc *streamdesc = (SQLStreamDesc *)streamDescriptor;

    if(streamdesc == 0) {
        clink.error().setRuntimeError(IFR_ERR_NULL_STREAMHANDLE_I, (IFR_Int4)this->m_index);
        DBUG_RETURN(IFR_NOT_OK);
    } else {
        if(streamdesc->StreamParam == 0) {
            clink.error().setRuntimeError(IFR_ERR_NULL_STREAMPARAM_I, (IFR_Int4)m_index);
        } else if(streamdesc->ReadProc == 0 && streamdesc->WriteProc == 0) {
            // set error for no procedures
        }
        if(streamdesc->StreamParam->Stream.hStream.memberCount != getPrecision()+1) {
            // This will only show up in trace
            clink.error().setRuntimeError(IFR_ERR_STREAM_MEMBERCOUNT_MISMATCH_III,
                                          (IFR_Int4)m_index,
                                          (IFR_Int4)streamdesc->StreamParam->Stream.hStream.memberCount,
                                          (IFR_Int4)(getPrecision() + 1));
            clink.error().clear();
        }
    }



    IFR_Retcode rc = datapart.addBinaryParameter((char *)streamdesc->StreamParam,
                                                 getIOLength() - 1,
                                                 m_shortinfo);
    if (rc == IFR_NOT_OK) {
        clink.error().setRuntimeError(IFR_ERR_CONVERSION_NOT_SUPPORTED_I, (IFR_Int4)this->m_index);
    }
    DBUG_RETURN(rc);
}

//----------------------------------------------------------------------
IFR_Retcode
IFRConversion_ABAPStreamConverter::putABAPStream(IFR_Parameter& parameter,
                                                 IFRPacket_DataPart& datapart,
                                                 IFR_Int4            requiredrows,
                                                 IFR_ConnectionItem& clink)
{
    DBUG_CONTEXT_METHOD_ENTER(IFRConversion_ABAPStreamConverter, putABAPStream, &clink);
    SQLStreamDesc *streamdesc = (SQLStreamDesc *)parameter.asChar();
    if(streamdesc->ReadProc == 0) {
        clink.error().setRuntimeError(IFR_ERR_STREAMHANDLE_NOTINPUT_I, (IFR_Int4)this->m_index);
        DBUG_RETURN(IFR_NOT_OK);
    }

    IFR_Int4 remainingBytes = datapart.getRemainingBytes() ;
// !!!
// For now, we must limit the ABAP stream size to 32KByte.
// !!!
#if 1
    if(remainingBytes > 32767) {
        remainingBytes = 32767;
    }
#endif
    void    *dataptr        = (void *)datapart.GetReadData(0);
    int      streamlength   = 0;
    int      rowsread       = requiredrows;

    int abap_rc = streamdesc->ReadProc(streamdesc->StreamParam,
                                       dataptr,
                                       remainingBytes,
                                       &streamlength,
                                       &rowsread);

    // If the function means that
    if(streamlength > remainingBytes) {
        clink.error().setMemoryAllocationFailed();
        DBUG_RETURN(IFR_NOT_OK);
    }

    switch(abap_rc) {
    case SQL_STREAM_NO_MORE_DATA:
        datapart.setLastPacket();
    case SQL_STREAM_OK:
        datapart.setArgCount(rowsread);
        datapart.setBufferLength(streamlength);
        DBUG_RETURN(IFR_OK);
    case SQL_STREAM_EXCEPTION:
    default:
        clink.error().setRuntimeError(IFR_ERR_STREAM_ERROR_INPUT_I, (IFR_Int4)this->m_index);
        DBUG_RETURN(IFR_NOT_OK);
    }
}

//----------------------------------------------------------------------
IFR_Retcode
IFRConversion_ABAPStreamConverter::getABAPStream(IFR_Parameter& parameter,
                                                 IFRPacket_DataPart& datapart,
                                                 IFR_Int4& abap_rc,
                                                 IFR_ConnectionItem& clink)
{
    DBUG_CONTEXT_METHOD_ENTER(IFRConversion_ABAPStreamConverter, putABAPStream, &clink);
    SQLStreamDesc *streamdesc = (SQLStreamDesc *)parameter.asChar();
    if(streamdesc->WriteProc == 0) {
        clink.error().setRuntimeError(IFR_ERR_STREAMHANDLE_NOTOUTPUT_I, (IFR_Int4)this->m_index);
        DBUG_RETURN(IFR_NOT_OK);
    }

    void *dataptr        = (void *)datapart.GetReadData(0);
    int   numberofrows   = datapart.getPartArguments();
    int   bufferlength   = datapart.getBufferLength();

    abap_rc = streamdesc->WriteProc(streamdesc->StreamParam,
                                    dataptr,
                                    numberofrows,
                                    bufferlength);

    switch(abap_rc) {
    case SQL_STREAM_OK:
        DBUG_RETURN(IFR_OK);
    default:
        clink.error().setRuntimeError(IFR_ERR_STREAM_ERROR_OUTPUT_I, (IFR_Int4)this->m_index);
        DBUG_RETURN(IFR_NOT_OK);
    }

}
